#!/usr/bin/env python3

import sys
from typing import Generator
from argparse import ArgumentParser
from collections import defaultdict
from dataclasses import dataclass, field

Q_MIN = SYN_MIN = CASES_MIN = 0
Q_MAX = SYN_MAX = CASES_MAX = 100


@dataclass
class Controller:
    """
    The main control class that loads and saves the files and creates Synonyms instances
    """

    input_file: str
    output_file: str
    cases: int = 0
    synonyms: list = field(default_factory=list)

    def load(self) -> None:
        with open(self.input_file, "r") as f:
            self._load_cases(f)

            for _ in range(self.cases):
                synonym = Synonym()
                self.synonyms.append(synonym)
                self._load_synonyms(f, synonym)
                self._load_queries(f, synonym)

    def _load_cases(self, f) -> None:
        cases = f.readline()

        try:
            self.cases = int(cases)
        except (ValueError, TypeError):
            sys.exit(
                f"The input file does not contain number of test cases, but got '{cases}'."
            )

        if self.cases < CASES_MIN or self.cases > CASES_MAX:
            sys.exit(
                f"Number of test cases should be between {CASES_MIN} and {CASES_MAX}, got '{self.cases}'."
            )

    def _load_synonyms(self, f, synonym) -> None:
        line = f.readline()

        try:
            synonyms_len = int(line)
        except (ValueError, TypeError):
            sys.exit(
                f"The input file does not contain number of synonym lines, but got '{line}'."
            )

        if synonyms_len < SYN_MIN or synonyms_len > SYN_MAX:
            sys.exit(
                f"Number of synonym lines should be between {SYN_MIN} and {SYN_MAX}, got '{synonyms_len}'."
            )

        for _ in range(synonyms_len):
            s1, s2 = map(str.lower, f.readline().strip().split())
            synonym.synonyms[s1].add(s2)
            synonym.synonyms[s2].add(s1)

    def _load_queries(self, f, synonym) -> None:
        line = f.readline()

        try:
            queries_len = int(line)
        except (ValueError, TypeError):
            sys.exit(
                f"The input file does not contain number of query lines, but got '{line}'."
            )

        if queries_len < Q_MIN or queries_len > Q_MAX:
            sys.exit(
                f"Number of query lines should be between {Q_MIN} and {Q_MAX}, got '{queries_len}'."
            )

        for _ in range(queries_len):
            synonym.queries.append(list(map(str.lower, f.readline().strip().split())))

    def save(self) -> None:
        with open(self.output_file, "w") as f:
            for synonym in self.synonyms:
                f.writelines(f"{line}\n" for line in synonym.solve())


@dataclass
class Synonym:
    """
    Resolve whether a query is a synonym or not
    """

    synonyms: dict = field(default_factory=lambda: defaultdict(set))
    queries: list = field(default_factory=list)

    def solve(self) -> Generator:
        for query in self.queries:
            yield "synonyms" if self.solve_query(*query) else "different"

    def solve_query(self, term1, term2, visited=None) -> bool:
        visited = visited or set()

        if term1 == term2:
            return True

        if term1 in visited:
            return False

        visited.add(term1)

        return any(
            map(
                lambda x: self.solve_query(x, term2, visited),
                self.synonyms.get(term1) or [],
            )
        )


def main() -> None:
    parser = ArgumentParser(description="Determine synonyms")
    parser.add_argument("input", help="input file")
    parser.add_argument("output", help="output file")
    args = parser.parse_args()

    c = Controller(args.input, args.output)
    c.load()
    c.save()


if __name__ == "__main__":
    main()
